package edu.stanford.nlp.io;

import java.io.*;
import java.util.*;

import edu.stanford.nlp.io.EncodingPrintWriter;
import edu.stanford.nlp.trees.Word;
import edu.stanford.nlp.process.*;

/**
 * An abstract class for a tokenizer implementing
 * edu.stanford.nlp.io.StreamTokenizer, based on a <code>Lexer</code>.
 *
 * <p> 
 *
 * This class is intended to be extended by adding a constructor that
 * passes the appropriate Lexer to the constructor of this class.  See
 * <code>Lexer</code> on suggestions for constructing an appropriate
 * implementing class.
 *
 * @author Roger Levy
 */
public abstract class AbstractStreamTokenizer implements edu.stanford.nlp.io.StreamTokenizer
{
  private Lexer l;

  public AbstractStreamTokenizer(Lexer l) {
    this.l = l;
  }

    /**
     * Returns the next token.    Satisfies the
     * <code>edu.stanford.nlp.io.StreamTokenizer</code> interface.
     * @return the next token as a String.  Returns null when end of
     * file has been reached.
     * @throws IOException
     */
    public String next() throws IOException {
	int a = l.IGNORE;
	while ((a = l.yylex()) == l.IGNORE); // skip tokens to be ignored
	if(a == l.getYYEOF()) {
	    return null;
	}
	else {
	    return l.yytext();
	}
    }

    /**
     * Pushes the previous token back.  Satisfies the
     * <code>edu.stanford.nlp.io.StreamTokenizer</code> interface.
     */
    public void pushBack() {
	l.pushBack(l.yytext().length());
    }

}
