package edu.stanford.nlp.io;

import java.io.*;

/** 
 * Static class for copying Files (oddly not supported by java.io.File).
 * @author Joseph Smarr (jsmarr@stanford.edu)
 */ 
public class FileCopier
{
    /** Private constructor to prevent direct instantiation. */
    private FileCopier() { }
        
    /**
     * Copies the contents of the given input file to the given output file.
     * Does nothing if inFile and outFile are the same file.
     * @throws FileNotFoundException if the input file can't be found.
     * @throws IOException if an I/O exception occurs while reading/writing.
     */
    public static void copyFile(File inFile,File outFile) throws FileNotFoundException,IOException
    {
        if(inFile.equals(outFile)) return; // bail out if they're equal
        
        FileInputStream fis=new FileInputStream(inFile);
        FileOutputStream fos=new FileOutputStream(outFile);
        byte[] buf=new byte[1024];
        int i=0;
        while((i=fis.read(buf))!=-1) fos.write(buf,0,i);
        fis.close();
        fos.close();
    }
}
