package edu.stanford.nlp.io;

/** A simple interface for streaming tokenizers.  This assumes that a
 *  token stream can be regarded as a sequence of things of type
 *  <code>String</code> which end at the end of file.  It avoids all the
 *  complexities of the <code>java.io.StreamTokenizer</code> class.
 *  <p>
 *  The expected constructors are one that takes a <code>Reader</code>,
 *  perhaps optionally one that takes an <code>InputStream</code>, and
 *  does the composition for the user.  There may also be other
 *  optional arguments that specify such things as whether eol is
 *  significant, or whether tokens should be lowercased.
 *  
 *  @author Christopher Manning
 *  @version 2003/01/09
 */
public interface StreamTokenizer {

    /** Returns the next token from the input Reader.  At end of file, it
     *	returns <code>null</code>.
     *  @return The token
     *  @throws IOException If there is some I/O error
     */
    public String next() throws java.io.IOException;

    /** Pushback the last read token from the tokenizer for re-reading.
     *  That is, the next call to <code>next()</code> will return the same
     *  token as the last call to <code>next()</code>.  This allows just
     *  a single level of lookahead in token reading.
     */
    public void pushBack();

}
