package edu.stanford.nlp.parser.lexparser;

import java.util.*;
import edu.stanford.nlp.trees.*;

/** A lexicon class for Chinese.  Extends the current Lexicon class,
 *  overriding its score and train methods to include a
 *  ChineseUnknownWordModel.
 * @author Roger Levy
 */
class ChineseLexicon extends Lexicon {
  ChineseUnknownWordModel unknown = new ChineseUnknownWordModel();

  private static final int STEPS = 1;
  private RandomWalk probRandomWalk;
  private static final boolean useRandomWalk = false;

  /* trains on a collection of trees */
  public void train(Collection trees) {
    super.train(trees);
    if(useRandomWalk)
      probRandomWalk = new RandomWalk(trees, STEPS);
    unknown.train(trees);
  }

  public double score(Object event, int loc) {
    IntTaggedWord iTW = (IntTaggedWord)event;

    double c_W = seenCounter.getCount(iTW);

    boolean seen = (c_W > 0.0);

    if(seen) {
      if(useRandomWalk)
	return scoreRandomWalk(iTW);
      else
	return super.score(event, loc);
    }
    else
      return unknown.score(iTW);
  }

  private double scoreRandomWalk(IntTaggedWord itw) {
    TaggedWord tw = itw.toTaggedWord();
    String word = tw.value();
    String tag = tw.tag();

    return probRandomWalk.score(tag,word);
  }

  private static long serialVersionUID = 221L;

}
