package edu.stanford.nlp.parser.lexparser;

import java.io.*;
import java.util.*;

import edu.stanford.nlp.process.*;
import edu.stanford.nlp.trees.*;
import edu.stanford.nlp.util.*;

/** A class for customizing the print method for a
 * edu.stanford.nlp.trees.<code>Tree</code> as the output of the
 * parser.  
 *
 * @author Roger Levy
 */  

class TreePrint {
  /* legal output tree formats */
  public static final String[] outputTreeFormats = { "penn","oneline","none" };

  private String format;
  private boolean lexicalize = false;
  private boolean printDepsSet = false;

  private HeadFinder hf = new CollinsHeadFinder();

  /** prints the tree. */
  public void printTree(Tree t, PrintWriter pw) {
    Tree outputTree = t;
    if(lexicalize) {
      Appliable a = TreeJugglers.getStringLabelTreeToCategoryWordTagTreeAppliable();
      outputTree = (Tree) a.apply(t);
      outputTree.percolateHeads(hf);
    }

    if(format.equals("penn"))
      outputTree.pennPrint(pw);
    else if(format.equals("oneline"))
      pw.println(outputTree.toString());

    if(printDepsSet) {
      Set depsSet = new DependencyEval("").makeObjects(outputTree);
      pw.println(depsSet.toString());
    }

  }


  public TreePrint(String format) {
    if (! Arrays.asList(outputTreeFormats).contains(format)) {
      this.format = "penn";
      throw new RuntimeException("Error: output tree format " + format + " not supported");
    }
    else
      this.format = format;
  }

  /** sets whether or not to lexicalize the output format of the
   * tree.*/
  public void setLexicalize(boolean b) {
    lexicalize = b;
  }

  /** Set whether or not to print the dependency set for the tree */
  public void setPrintDepsSet(boolean b) {
    printDepsSet = b;
  }

  /** Set the headfinder to be used for lexicalizing trees.  Relevant
      if the output is to be lexicalized.  By default,
      <code>CollinsHeadFinder</code> is used. */
  public void setHeadFinder(HeadFinder hf) { 
    this.hf  = hf; 
  }

}
