package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.trees.*;
import java.io.*;
import java.util.*;


/** Contains fields necessary to get the parser to parse an arbitrary
 *  treebank.
 *
 *  @author Roger Levy
 *  @version   03/05/2003
 */

public interface TreebankLangParserParams {

  public HeadFinder headFinder();


  public void setInputEncoding(String encoding);


  public void setOutputEncoding(String encoding);

  /** reads in trees from the source you want.  It's the
   * responsibility of tr to deal properly with character-set encoding
   * of the input.  It also is the responsibility of tr to properly
   * normalize trees */
  //public TreeReader treeReader();

  public Lexicon lex();

  /** the tree transformer used to produce trees for evaluation.  Will be applied both to the */

  public TreeTransformer collinizer();
  public TreeTransformer collinizerEvalb();

  /** returns a MemoryTreebank appropriate to the treebank source */
  public MemoryTreebank memoryTreebank();

  /** returns a MemoryTreebank appropriate to the testing treebank source */
  public MemoryTreebank testMemoryTreebank();

  /** returns a TreebankLanguagePack containing Treebank-specific (but
   * not parser-specific) info such as what is punctuation, and also
   * information about the structure of labels */
  public TreebankLanguagePack treebankLanguagePack();

  /** returns a PrintWriter used to print output. It's the
   * responsibility of the returned PrintWriter to deal properly with
   * character encodings for the relevant treebank */
  public PrintWriter pw();

  /** returns a PrintWriter used to print output to the OutputStream
   * o. It's the responsibility of the returned PrintWriter to deal
   * properly with character encodings for the relevant treebank */
  public PrintWriter pw(OutputStream o);


  /** Returns the splitting strings used for selective splits.
   *  @return An array containing ancestor-annotated Strings: categories
   *    should be split according to these ancestor annotations.
   */
  public String[] splitters();

  /** Returns the splitting strings used for selective splits.
   *  @return An array containing ancestor-annotated Strings: categories
   *    should be split according to these ancestor annotations.
   */
  public String[] sisterSplitters();


  /** transformTree does language-specific tree transformations such
   *  as splicing. Any parameterizations should be inside the specific
   * TreebankLangParserParams class 
   */
  public TreeHeadPair transformTree(Tree t, Tree root, TreeHeadPair thp);

  /** display language-specific settings */
  public void display();

  /** Set language-specific options according to flags.
   *  This routine should process the option starting in args[i] (which
   *  might potentially be several arguments long if it takes arguments).
   *  It should return the index after the last index it consumed in
   *  processing.  In particular, if it cannot process the current option,
   *  the return value should be i.
   */
  public int setOptionFlag(String[] args, int i);

}
