package edu.stanford.nlp.process;

import java.io.*;
import java.util.*;

/**
 * Abstract tokenizer.  Tokenizers extending AbstractTokenizer need only
 * implement <code>next()</code>, <code>hasNext()</code>, and <code>setSource(Reader r)</code>. 
 *
 * @author Teg Grenager (grenager@stanford.edu)
 */

public abstract class AbstractTokenizer implements Tokenizer {
    
    /** Returns the next token from this Tokenizer. */
    public abstract Object next();

    /** Returns <code>true</code> if this Tokenizer has more elements. */
    public abstract boolean hasNext();

    /**
     *
     */
    public void remove() {
	throw new UnsupportedOperationException();
    }
    
    /**
     *
     */
    public void pushBack() {
	throw new UnsupportedOperationException();
    }
    
    /**
     * Returns text as a List of tokens.  
     */
    public List tokenize() {
	List result = new ArrayList(1000);
	while (hasNext()) {
	    result.add(next());
	}
	return result;
    }

    /** Sets the source for this Tokenizer. */
    public abstract void setSource(Reader r);

}


