package edu.stanford.nlp.process;

import java.util.*;
import edu.stanford.nlp.dbm.*;

/**
 * Processor that takes an Appliable and applies to every element in
 * the input Document. This is useful when you want to transform
 * each element in an isolated way.
 * @author Sepandar Kamvar (sdkamvar@stanford.edu)
 */
public class DocumentProcessor implements Processor 
{
    /** Appliable used to transform each document element during processing. */
    protected Appliable appliable;

    /** Does nothing but allows subclasses with empty constructor. */
    protected DocumentProcessor() {}

    /**
     * Instantiates a new DocumentProcessor for a given Appliable
     */ 
    public DocumentProcessor(Appliable a) { appliable=a; }

    /** 
     * Converts a Document to a different Document, by transforming
     * or filtering the elements in <tt>in</tt> using the <tt>Appliable</tt>
     * given in the constructor. 
     */
    public Document process(final Document in) 
    {
        Document out=in.blankDocument(); // copies the meta data from in
        Iterator iter=in.iterator();
        while(iter.hasNext())
            out.add(appliable.apply(iter.next()));

        return(out);
    }
}

