package edu.stanford.nlp.process;


import java.io.*;
import java.util.*;

import edu.stanford.nlp.trees.Word;

/**
 * Tokenizer implementation that conforms to the Penn Treebank tokenization
 * conventions.
 * This tokenizer is a Java implementation of Professor Chris Manning's Flex
 * tokenizer, pgtt-treebank.l.  It reads raw text and outputs a List of
 * tokens in the Penn treebank format. It can optionally return carriage
 * returns as tokens.
 * 
 * @author Teg Grenager (grenager@stanford.edu)
 */
public class DummyTokenizer extends AbstractTokenizer
{
    DummyLexer lexer;
    Word nextToken;

    public DummyTokenizer() {}

    /** Constructs a new DummyTokenizer that treats carriage returns as normal whitespace. */
    public DummyTokenizer(Reader r) { 
	setSource(r);
    }
    
    /**
     * Get the next valid word from the lexer if possible.
     */
    private void getNext() {
	if (lexer == null) {
	    nextToken = null;
	    return;
	}
	try {
	    nextToken = lexer.next();
	} catch (Exception e) {
	    nextToken = null;
	}
    }

    /**
     *
     */
    public boolean hasNext() {
	return nextToken != null;
    }
    
    /**
     *
     */
    public Object next() {
	Word result = nextToken;
	getNext();
	return result;
    }


    /** Reads a file from the argument and prints its tokens one per line.
     *  This is mainly as a testing aid, but it can also be quite useful 
     *  standalone to turn a corpus into a one token per line file of tokens.
     *  <p>
     *  Usage: <code>java edu.stanford.nlp.process.DummyTokenizer filename
     *  </code>
     *  @param args Command line arguments
     */
    public static void main (String[] args) throws IOException {
    if (args.length < 1) {
        System.err.println("usage: java edu.stanford.nlp.process." +
			   "DummyTokenizer [-cr] filename");
        return;
    }
    DummyTokenizer tokenizer = new DummyTokenizer(new FileReader(args[args.length-1]));
    List words = tokenizer.tokenize();
    for (int i=0; i < words.size(); i++)
        System.out.println(words.get(i));
    }

    public void setSource(Reader r) {
	lexer = new DummyLexer(r);
	getNext();
    }

}

