package edu.stanford.nlp.process;

import java.util.*;
import java.io.*;
import edu.stanford.nlp.trees.*;

%%

%class PTBLexer
%standalone
%unicode
%function next
%type Word

%{
  final static public String opendblquote = "``";
  final static public String closedblquote = "''";
  final static public String openparen = "-LRB-";
  final static public String closeparen = "-RRB-";
  final static public String openbrace = "-LCB-";
  final static public String closebrace = "-RCB-";
  public static final String ptbmdash = "--";
  public static final String cr = "*CR*"; // js: for tokenizing carriage returns

  /** This quotes a character with a backslash, but doesn't do it
   *  if the character is already preceded by a backslash
   */
  static private String delimit (String s, char c) {
    int i = s.indexOf(c);
    while (i != -1) {
      if (i == 0 || s.charAt(i - 1) != '\\') {
        s = s.substring(0, i) + "\\" + s.substring(i);
        i = s.indexOf(c, i + 2);
      } else {
        i = s.indexOf(c, i + 1);
      } 
    }
    return s;
  }

%}

SGML = <\/?[A-Za-z!][^>]*>
SPMDASH = &(MD|mdash);
SPPUNC = &(MD|HT|TL|UR|LR|QC|QL|QR|odq|cdq|mdash|lt|gt|amp|AMP|#[0-9]+);
SPLET = &[aeiou](acute|grave|uml);
SPACE = [ \t]+
SPACENL = [ \t\n]+
SENTEND = [ \t\n][ \t\n]+|[ \t\n]+([A-Z]|{SGML})
DIGIT = [0-9]
DATE = {DIGIT}{1,2}[\-\/]{DIGIT}{1,2}[\-\/]{DIGIT}{2,4}
NUM = {DIGIT}+|{DIGIT}*([\.:,]{DIGIT}+)+
NUMBER = [\-+]?{NUM}|\({NUM}\)
FRAC = {DIGIT}+\\?\/{DIGIT}+
DOLSIGN = ([A-Z]*\$|#)
/* not used DOLLAR	{DOLSIGN}[ \t]*{NUMBER}  */
/* |\( ?{NUMBER} ?\))	 # is for pound signs */
WORD = ([A-Za-z]|{SPLET})+
/* The $ was for things like New$ */
/* WAS: only keep hyphens with short one side like co-ed */
/* But treebank just allows hyphenated things as words! */
THING = [A-Za-z0-9][A-Za-z0-9]*[A-Za-z0-9]|[A-Z][A-Z&+_;-]+[A-Z+]
THING2 = [A-Za-z0-9%.,]+(-[A-Za-z0-9]+)+|{ACRO}\.-[A-Za-z]+
THING3 = [A-Za-z0-9]+\\?\/[A-Za-z0-9]+(-[A-Za-z0-9]+)?
APOS = [']
HTHING = {THING2}|[dDOlL]{APOS}{THING}
REDAUX = {APOS}([m|M]|re|RE|[sS]|ve|VE|[Dd]|ll|LL)
/* For things that will have n't on the end */
SWORD = [A-Za-z]*[A-MO-Za-mo-z]
SREDAUX = [nN]{APOS}[tT]
/* Tokens you want but already okay: C'mon 'n' '[2-9]0s '[eE]m 'till?
   [Yy]'all 'Cause Shi'ite B'Gosh o'clock.  Here now only need apostrophe
   final words. 'n' is a token from dunno where. */
APOWORD = {APOS}n{APOS}|Dunkin{APOS}|[Ss]omethin{APOS}|ol{APOS}|{APOS}[Ee]m|C{APOS}[Mm][Oo][Nn]|{APOS}[2-9]0s|{APOS}till?|o{APOS}clock|Shi{APOS}ite|{APOS}[cC]ause
FULLURL = http:\/\/[^ \t\n\f\r\"<>|()]+[^ \t\n\f\r\"<>|.!?(){},-]

/* Abbreviations - induced from 1987 WSJ by hand */
ABMONTH = Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|Sept|Oct|Nov|Dec
/* Jun and Jul barely occur, but don't seem dangerous */
ABDAYS = Mon|Tue|Tues|Wed|Thu|Thurs|Fri|a\.m|p\.m
/* Sat. and Sun. barely occur and can easily lead to errors, so we omit them */
ABST1 = Calif|Mass|Conn|Fla|Ill|Mich|Pa|Va|Ariz|Tenn|Mo|Md|Wis|Minn|Ind|Okla
ABST2 = Wash|Kan|Ore|Ga|Colo|Ky|Del|Ala|La|Nev|Neb|Ark|Miss|Vt|Wyo|Tex
ABSTATE = {ABST1}|{ABST2}
ACRO = [A-Za-z](\.[A-Za-z])+|(Canada|Sino|Korean|Japan|non)-U\.S|U\.S\.-(U\.K|U\.S\.S\.R)
ABLIST = [a-e]
ABTI1 = Mr|Mrs|Ms|Dr|Prof|Sen|Reps?|Lt|Col|Gen|Messrs|Gov|Adm|Rev|Maj|Sgt
ABTI2 = Capt|Sens|St|Ave
ABTITLE = {ABTI1}|{ABTI2}
ABPTIT = Jr|Bros|Sr
ABCOMP = Inc|INC|Co|CO|Cos|COS|Corp|CORP|Pty|Ltd|Plc
/* What about capitalization?  CORP. etc. */
ABNUM = p|No
/* ABBREV1 abbreviations are normally followed by lower case words.  If
   they're followed by an uppercase one, we assume there is also a
   sentence boundary */
ABBREV3 = {ABMONTH}|{ABDAYS}|{ABSTATE}|{ABCOMP}|{ABNUM}|{ABPTIT}|[Ee]tc|Ph
ABBREV1 = {ABBREV3}\.
/* ABRREV2 abbreviations are normally followed by an upper case word.  We
   assume they aren't used sentence finally */
/* ACRO Is a bad case -- can go either way! */
ABBREV4	= [A-Z]|{ABTITLE}|{ABLIST}|vs|v|Alex|Cie|a.k.a|TREAS|{ACRO}
ABBREV2	= {ABBREV4}\.
/* Cie. is used before French companies */
/* in the WSJ Alex. is generally an abbreviation for Alex. Brown, brokers! */
/* In tables: Mkt. for market Div. for division of company, Chg., Yr.: year */

PHONE = \([0-9]{3}\)\ ?[0-9]{3}[\- ][0-9]{4}
OPBRAC = [<\[]
CLBRAC = [>\]]
HYPHENS = \-+
LDOTS = \.{3,5}|(\.\ ){2,4}\.
ATS = @+
UNDS = _+
ASTS = \*+|(\\*)+
HASHES = #+
FNMARKS = {ATS}|{HASHES}|{UNDS}
INSENTP =[,;:]
QUOTES =`|{APOS}|``|''
DBLQUOT = \"|&quot;
TBSPEC = \-(RRB|LRB|RCB|LCB|RSB|LSB)\-|S&Ls

%%

{SGML}			{ return new Word(yytext()); }
{SPMDASH}		{ return new Word(ptbmdash); }
{SPPUNC}		{ return new Word(yytext()); }
{WORD}/{REDAUX}		{ return new Word(yytext()); }
{SWORD}/{SREDAUX}	{ yypushback(1); return new Word(yytext()); }
{WORD}			{ return new Word(yytext()); }
{APOWORD}		{ return new Word(yytext()); }
{FULLURL}		{ return new Word(yytext()); }
{REDAUX}		{ return new Word(yytext()); }
{SREDAUX}		{ return new Word(yytext()); }
{DATE}			{ return new Word(yytext()); }
{NUMBER}		{ return new Word(yytext()); }
{FRAC}			{ return new Word(delimit(yytext(), '/')); }
{THING3}		{ return new Word(delimit(yytext(), '/')); }
{DOLSIGN}		{ return new Word(yytext()); }
{ABBREV1}/{SENTEND}	{ return new Word(yytext()); }
{ABBREV1}		{ return new Word(yytext()); }
{ABBREV2}		{ return new Word(yytext()); }
{ABBREV3}/{SPACE}	{ return new Word(yytext()); }
{ABBREV4}/{SPACE}	{ return new Word(yytext()); }
{ACRO}/{SPACENL}	{ return new Word(yytext()); }
{TBSPEC}/{SPACENL}	{ return new Word(yytext()); }
{WORD}\./{INSENTP}	{ return new Word(yytext()); }
{PHONE}			{ return new Word(yytext()); }
{DBLQUOT}/[A-Za-z0-9$]	{ return new Word(opendblquote); }
{DBLQUOT}		{ return new Word(closedblquote); }
{QUOTES}		{ return new Word(yytext()); }
\+		{ return new Word(yytext()); }
%|&		{ return new Word(yytext()); }
\~|\^		{ return new Word(yytext()); }
\||\\|0x7f	{ /* ignore */ }
{OPBRAC}	{ return new Word(yytext()); }
{CLBRAC}	{ return new Word(yytext()); }
\{		{ return new Word(openbrace); }
\}		{ return new Word(closebrace); }
\(		{ return new Word(openparen); }
\)		{ return new Word(closeparen); }
{HYPHENS}	{ return new Word(yytext()); }
{LDOTS}		{ return new Word(yytext()); }
{FNMARKS}	{ return new Word(yytext()); }
{ASTS}		{ return new Word(delimit(yytext(), '*')); }
{INSENTP}	{ return new Word(yytext()); }
\.|\?|\!	{ return new Word(yytext()); }
=		{ return new Word(yytext()); }
\/		{ return new Word(yytext()); }
{HTHING}/[^a-zA-Z0-9&+]	{ return new Word(yytext()); }
{THING}		{ return new Word(yytext()); }
\0|{SPACE}	{ /* ignore */ }
\n|\r		{ return(new Word(cr)); } // js: for tokenizing carriage returns
&nbsp;		{ /* ignore */ }
.		{ System.err.println(yytext()); }
<<EOF>> { return null; }
