package edu.stanford.nlp.process;

import java.util.*;

import edu.stanford.nlp.dbm.Document;
import edu.stanford.nlp.trees.Sentence;
import edu.stanford.nlp.trees.Word;

/**
 * Transforms a Document of Sentences to a Document of Words
 * by flattening out the Sentences. All the words in each 
 * Sentence are dumped out into a new Document with the same
 * meta-data as the original Document. Opposite of 
 * {@link WordToSentenceProcessor}.
 *
 * @author Joseph Smarr (jsmarr@stanford.edu)
 */
public class SentenceToWordProcessor implements Processor
{
    /** 
     * Transforms the given Document of Sentences into a 
     * Document of Words by flattening out the Sentences.
     */
    public Document process(Document sentences)
    {
        Document words=sentences.blankDocument();
        for(Iterator iter=sentences.iterator();iter.hasNext();)
            words.addAll((Collection)iter.next()); // sentence is just a list
        return(words);
    }
}

