package edu.stanford.nlp.process;

import java.io.*;
import java.util.*;
import edu.stanford.nlp.trees.Word;

/**
 *  Simple stoplist class.
 * @author Sepandar Kamvar
**/

public class StopList
{
  private HashSet wordSet;

  /*
	 *     Constructs a stoplist with very few stopwords.
  */

  public StopList()
  {
    wordSet = new HashSet();
    addGenericWords();
  }
  
  /**
     Constructs a new stoplist from the contents of a file. It is
     assumed that the file contains stopwords, one on a line.
     The stopwords need not be in any order.
  **/

  public StopList(File list) 
  {
			wordSet = new HashSet();

			try{
					BufferedReader reader = new BufferedReader(new FileReader(list));
					
					while (reader.ready())
							wordSet.add(new Word(reader.readLine()));
			} catch(IOException e) {
					e.printStackTrace(System.err);
					addGenericWords();
			}
	}

  /**
     Adds some extremely common words to the stoplist.
  **/
  private void addGenericWords()
  {
			String[] genericWords= {"a", "an", "the", "and", "or", "but", "nor"};
			for (int i=1; i<7; i++)
					wordSet.add(new Word(genericWords[i]));
  }
  
  /**
     Returns true if the word is in the stoplist.
  **/
  public boolean contains(Word word)
  {
    return wordSet.contains(word);
  }

 /**
     Returns true if the word is in the stoplist.
  **/
  public boolean contains(String word)
  {
    return wordSet.contains(new Word(word));
  }


}
