package edu.stanford.nlp.process;
import java.util.*;
import java.io.*;
import edu.stanford.nlp.trees.Word;
import edu.stanford.nlp.dbm.*;


/**
 * Filter which removes stop-listed words.
 *

 *
@author Sepandar Kamvar (sdkamvar@stanford.edu)
 */

public class StoplistFilter implements Processor {
		
		private StopList stoplist;
		
		/**
		 * Create a new StopListFilter with a small default stoplist
		 */
		public StoplistFilter() { this(new StopList()); }
		
		/**
		 * Create a new StopListFilter with the stoplist given in <code>stoplistfile</code>
		 */
		public StoplistFilter(String stoplistfile) {
				this(new StopList(new File(stoplistfile)));
		}

      /** Create a new StoplistFilter with the given StopList. */
      public StoplistFilter(StopList stoplist) { this.stoplist=stoplist; }
		
      /**
       * Returns a new Document with the same meta-data as <tt>in</tt> and the same words 
       * except those on the stop list this filter was constructed with.
       */
		public Document process(Document in) 
		{
			Document out=in.blankDocument();
			Iterator iter=in.iterator();
			while(iter.hasNext())
			{
				Word w=(Word)iter.next();
				if(!stoplist.contains(w)) out.add(w);
			}
			return(out);
		}
}

