package edu.stanford.nlp.process;
import java.util.*;
import java.io.*;

/**
 * Tokenizers break up text into individual Objects. These objects may be Strings, Words, or 
 * other Objects. This Tokenizer interface allows the source to be set upon construction, or
 * to be reset using the <code>setSource(Reader r)</code> method. Thus each Tokenizer instance
 * may be used for one data source or several.
 *
 * @author Teg Grenager (grenager@stanford.edu)
 */
public interface Tokenizer extends Iterator {    

    /** Returns the next token from this Tokenizer. */
    public abstract Object next();

    /** Returns <code>true</code> if this Tokenizer has more elements. */
    public abstract boolean hasNext();

    /** 
     * Removes from the underlying collection the last element returned by 
     * the iterator (optional operation). This method can be called only once 
     * per call to next. 
     */
    public abstract void remove();

    /**
     * Pushes the last token returned back on this Tokenizer, so that it will
     * be returned again in the next call to <code>next()</code>.
     */
    public abstract void pushBack();

    /** Returns all tokens of this Tokenizer as a List for convenience. */
    public abstract List tokenize();

    /** Sets the source for this Tokenizer. */
    public abstract void setSource(Reader r);

}
