package edu.stanford.nlp.process;

import java.util.*;

%%

%class PTBLexer
%standalone
%unicode

%{
  final static public String opendblquote = "``";
  final static public String closedblquote = "''";
  final static public String openparen = "-LRB-";
  final static public String closeparen = "-RRB-";
  final static public String openbrace = "-LCB-";
  final static public String closebrace = "-RCB-";

  /** This quotes a character with a backslash, but doesn't do it
   *  if the character is already preceded by a backslash
   */
  static private String delimit (String s, char c) {
    int i = s.indexOf(c);
    while (i != -1) {
      if (i == 0 || s.charAt(i - 1) != '\\'; {
        s = s.substring(0, i) + "\\" + s.substring(i);
        i = s.indexOf(c, i + 2);
      } else {
        i = s.indexOf(c, i + 1);
      } 
    }
    return s;
  }

  public List words = null;

  public void Process (String s) {
    if (words == null) System.out.print(s + " ");
    else words.add(s);
  }
%}

SGML = <\/?[A-Za-z!][^>]*>
SPPUNC = &(MD|HT|TL|UR|LR|QC|QL|QR|odq|cdq|mdash|lt|gt|amp|AMP|#[0-9]+);
SPLET = &[aeiou](acute|grave|uml);
SPACE = [ \t]+
SPACENL = [ \t\n]+
SENTEND = [ \t\n][ \t\n]+|[ \t\n]+([A-Z]|{SGML})
DIGIT = [0-9]
DATE = {DIGIT}{1,2}[\-\/]{DIGIT}{1,2}[\-\/]{DIGIT}{2,4}
NUM = {DIGIT}+|{DIGIT}*([\.:,]{DIGIT}+)+
NUMBER = [\-+]?{NUM}|\({NUM}\)
FRAC = {DIGIT}+\\?\/{DIGIT}+
DOLSIGN = ([A-Z]*\$|#)
/* not used DOLLAR	{DOLSIGN}[ \t]*{NUMBER}  */
/* |\( ?{NUMBER} ?\))	 # is for pound signs */
WORD = ([A-Za-z]|{SPLET})+
/* The $ was for things like New$ */
/* WAS: only keep hyphens with short one side like co-ed */
/* But treebank just allows hyphenated things as words! */
THING = [A-Za-z0-9][A-Za-z0-9]*[A-Za-z0-9]|[A-Z][A-Z&+_;-]+[A-Z+]
THING2 = [A-Za-z0-9%.,]+(-[A-Za-z0-9]+)+|{ACRO}\.-[A-Za-z]+
THING3 = [A-Za-z0-9]+\\?\/[A-Za-z0-9]+(-[A-Za-z0-9]+)?
HTHING = {THING2}|[dDOlL]'{THING}
REDAUX = 'm|'re|'s|'ve|'d|'ll|'|'S
/* For things that will have n't on the end */
SWORD = [A-Za-z]*[A-MO-Za-mo-z]
SREDAUX = n't|N'T
NTWORD = {SWORD}{SREDAUX}
/* Tokens you want but already okay: C'mon 'n' '[2-9]0s '[eE]m 'till?
   [Yy]'all 'Cause Shi'ite B'Gosh o'clock.  Here now only need apostrophe
   final words. 'n' is a token from dunno where. */
APOWORD = Dunkin'|somethin'|ol'|'[Ee]m
FULLURL = http:\/\/[^ \t\n\f\r\"<>|()]+[^ \t\n\f\r\"<>|.!?(){},-]

/* Abbreviations - induced from 1987 WSJ by hand */
ABMONTH = Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|Sept|Oct|Nov|Dec
/* Jun and Jul barely occur, but don't seem dangerous */
ABDAYS = Mon|Tue|Tues|Wed|Thu|Thurs|Fri|a\.m|p\.m
/* Sat. and Sun. barely occur and can easily lead to errors, so we omit them */
ABST1 = Calif|Mass|Conn|Fla|Ill|Mich|Pa|Va|Ariz|Tenn|Mo|Md|Wis|Minn|Ind|Okla
ABST2 = Wash|Kan|Ore|Ga|Colo|Ky|Del|Ala|La|Nev|Neb|Ark|Miss|Vt|Wyo|Tex
ABSTATE = {ABST1}|{ABST2}
ACRO = [A-Za-z](\.[A-Za-z])+|non-U.S
ABLIST = [a-e]
ABTI1 = Mr|Mrs|Ms|Dr|Prof|Sen|Reps?|Lt|Col|Gen|Messrs|Gov|Adm|Rev|Maj|Sgt
ABTI2 = Capt|Sens|St|Ave
ABTITLE = {ABTI1}|{ABTI2}
ABPTIT = Jr|Bros|Sr
ABCOMP = Inc|INC|Co|Cos|Corp|CORP|Pty|Ltd|Plc
/* What about capitalization?  CORP. etc. */
ABNUM = p|No
/* ABBREV1 abbreviations are normally followed by lower case words.  If
   they're followed by an uppercase one, we assume there is also a
   sentence boundary */
ABBREV3 = {ABMONTH}|{ABDAYS}|{ABSTATE}|{ABCOMP}|{ABNUM}|{ABPTIT}|[eE]tc
ABBREV1 = {ABBREV3}\.
/* ABRREV2 abbreviations are normally followed by an upper case word.  We
   assume they aren't used sentence finally */
/* ACRO Is a bad case -- can go either way! */
ABBREV4	= [A-Z]|{ABTITLE}|{ABLIST}|vs|v|Alex|Cie|a.k.a|TREAS|{ACRO}
ABBREV2	= {ABBREV4}\.
/* Cie. is used before French companies */
/* in the WSJ Alex. is generally an abbreviation for Alex. Brown, brokers! */
/* In tables: Mkt. for market Div. for division of company, Chg., Yr.: year */

PHONE = \([0-9]{3}\)\ ?[0-9]{3}[\- ][0-9]{4}
OPBRAC = [<\[]
CLBRAC = [>\]]
HYPHENS = \-+
LDOTS = \.{3,5}|(\.\ ){2,4}\.
ATS = @+
UNDS = _+
ASTS = \*+
HASHES = #+
FNMARKS = {ATS}|{HASHES}|{UNDS}
INSENTP =[,;:]
QUOTES =`|'|``|''
DBLQUOT = \"|&quot;
TBSPEC = \-(RRB|LRB|RCB|LCB)\-|S&Ls

%%

{SGML}			{ Process(yytext()); }
{SPPUNC}		{ Process(yytext()); }
{WORD}/{REDAUX}		{ Process(yytext()); }
{NTWORD}	/* special jflex treatment as lookahead didn't work */
			{ Process(yytext().substring(0, yytext().length()-3));
			  Process(yytext().substring(yytext().length() - 3)); }
{REDAUX}		{ Process(yytext()); }
{SREDAUX}		{ Process(yytext()); }
{WORD}			{ Process(yytext()); }
{APOWORD}		{ Process(yytext()); }
{FULLURL}		{ Process(yytext()); }
{DATE}			{ Process(yytext()); }
{NUMBER}		{ Process(yytext()); }
{FRAC}			{ Process(delimit(yytext(), '/')); }
{THING3}		{ Process(delimit(yytext(), '/')); }
{DOLSIGN}		{ Process(yytext()); }
{ABBREV1}/{SENTEND}	{ Process(yytext()); }
{ABBREV1}		{ Process(yytext()); }
{ABBREV2}		{ Process(yytext()); }
{ABBREV3}/{SPACE}	{ Process(yytext()); }
{ABBREV4}/{SPACE}	{ Process(yytext()); }
{ACRO}/{SPACENL}	{ Process(yytext()); }
{TBSPEC}/{SPACENL}	{ Process(yytext()); }
{WORD}\./{INSENTP}	{ Process(yytext()); }
{PHONE}			{ Process(yytext()); }
{DBLQUOT}/[A-Za-z0-9$]	{ Process(opendblquote); }
{DBLQUOT}		{ Process(closedblquote); }
{QUOTES}		{ Process(yytext()); }
\+		{ Process(yytext()); }
%|&		{ Process(yytext()); }
\~|\^		{ Process(yytext()); }
\||\\|0x7f	{ /* ignore */ }
{OPBRAC}	{ Process(yytext()); }
{CLBRAC}	{ Process(yytext()); }
\{		{ Process(openbrace); }
\}		{ Process(closebrace); }
\(		{ Process(openparen); }
\)		{ Process(closeparen); }
{HYPHENS}	{ Process(yytext()); }
{LDOTS}		{ Process(yytext()); }
{FNMARKS}	{ Process(yytext()); }
{ASTS}		{ Process(delimit(yytext(), '*')); }
{INSENTP}	{ Process(yytext()); }
\.|\?|\!|_	{ Process(yytext()); }
=		{ Process(yytext()); }
\/		{ Process(yytext()); }
{HTHING}/[^a-zA-Z0-9&+]	{ Process(yytext()); }
{THING}		{ Process(yytext()); }
\0|{SPACE}	{ /* ignore */ }
\n|\r		{ /* ignore */ }
&nbsp;		{ /* ignore */ }
.		{ System.err.println("error: \"" + yytext() + "\""); }
