package edu.stanford.nlp.trees;

import java.io.Reader;

/** This class implements a <code>TreeReaderFactory</code> that produces
 *  labeled, scored array-based Trees, which have been cleaned up to 
 *  delete empties, etc.  This seems to be a common case.
 *  @author Christopher Manning
 */
public class ParentTransformedLabeledNormalizedTreeReaderFactory 
    implements TreeReaderFactory {

    /** An implementation of the <code>TreeReaderFactory</code> interface.
     *  It creates a <code>TreeReader</code> which normalizes trees using
     *  the <code>BobChrisTreeNormalizer</code>, and makes
     *  <code>LabeledScoredTree</code> objects with 
     *  <code>CategoryWordTag</code> labels.
     */
    public TreeReader newTreeReader(Reader in) {
	return new PennTreeReader(in, 
			      new LabeledScoredTreeFactory(new WordFactory()),
				  TreeNormalizers.parentTransformedBobChris());
    }

}
