package edu.stanford.nlp.trees;

/** ??
 *  @author Dan Klein
 */
public class PruneNodesStripSubtagsTreeNormalizer extends TreeNormalizer {

    /** Normalizes a leaf contents.
     *  This implementation interns the leaf.
     */
    public String normalizeTerminal(String leaf) {
	// We could unquote * and / with backslash \ in front of them
	return leaf.intern();
    }


    /** Normalizes a nonterminal contents.
     *  This implementation strips functional tags, etc. and interns the
     *  nonterminal.
     */
    public String normalizeNonterminal(String category) {
	return cleanUpLabel(category).intern();
    }


    /** Normalize a whole tree -- one can assume that this is the root.
     *  This implementation deletes empty elements (ones with nonterminal
     *  labels passing the filter) from the tree
     */
  
  private edu.stanford.nlp.util.Filter nodeFilter;

  public Tree normalizeWholeTree(Tree tree, TreeFactory tf) {
    return tree.prune(nodeFilter, tf);
  }


    /** Remove things like hyphened functional tags and equals from the
     *  end of a node label
     */
    private static String cleanUpLabel(String label) {
	if (label == null) {
	    label = "ROOT";
	    // String constants are always interned
	} else {
	    // a '-' at the beginning of label is okay (punctuation tag!)
	    int k = label.indexOf('-');
	    if (k > 0) {
		label = label.substring(0, k);
	    }
	    k = label.indexOf('=');
	    if (k > 0) {
		label = label.substring(0, k);
	    }
	    k = label.indexOf('|');
	    if (k > 0) {
		label = label.substring(0, k);
	    }
	}
	return label;
    }

  public PruneNodesStripSubtagsTreeNormalizer(edu.stanford.nlp.util.Filter filter) {
    nodeFilter = filter;
  }

}
