package edu.stanford.nlp.trees.international;

import java.io.*;
import java.util.*;

import edu.stanford.nlp.io.EncodingPrintWriter;
import edu.stanford.nlp.trees.Word;
import edu.stanford.nlp.process.*;

/**
 * A simple tokenizer for tokenizing Penn Chinese Treebank files.  A
 * token is any parenthesis, node label, or terminal.  All SGML
 * content of the files is ignored.
 *
 * @author Roger Levy
 * @version 01/17/2003
 */
public class CHTBTokenizer implements edu.stanford.nlp.io.StreamTokenizer
{
    private CHTBLexer l = null;

    /**
     * Constructs a new tokenizer from a Reader.  Note that getting
     * the bytes going into the Reader into Java-internal Unicode is
     * not the tokenizer's job.  This can be done by converting the
     * file with <code>ConvertEncodingThread</code>, or by specifying
     * the files encoding explicitly in the Reader with
     * java.io.<code>InputStreamReader</code>.
     * @param r Reader
     */
    public CHTBTokenizer(Reader r) {
	l = new CHTBLexer(r);
    }


    /**
     * Returns the next token.  Satisfies the
     * <code>edu.stanford.nlp.io.StreamTokenizer</code> interface.
     * @return the next token as a String.
     * @throws IOException
     */
    public String next() throws IOException {
	int a = 0;
	while ((a = l.yylex()) == l.IGNORE);
	if(a == l.YYEOF) {
	    return null;
	}
	else {
	    return l.match();
	}
    }

    /**
     * Pushes the previous token back.  Satisfies the
     * <code>edu.stanford.nlp.io.StreamTokenizer</code> interface.
     */
    public void pushBack() {
	l.pushback(l.yytext().length());
    }


    /**
     * The main() method tokenizes a file in the specified Encoding
     * and prints it to standard output in the specified Encoding.
     * Its arguments are (Infile, Encoding).
     */
    public static void main(String[] args) throws IOException {
      
      String encoding = args[1];
      Reader in = new BufferedReader(new InputStreamReader(new FileInputStream(args[0]),encoding));
      
      edu.stanford.nlp.io.StreamTokenizer st = new CHTBTokenizer(in);
      
      String s;
      while ((s = st.next()) != null) {
	EncodingPrintWriter.out.println(s, encoding);
	// EncodingPrintWriter.out.println("|" + s + "| (" + s.length() + ")",
	//				encoding);
      }
    }

}
