package edu.stanford.nlp.trees.international;

import java.io.*;

/**
 * Converts the encoding of an <code>InputStream</code> and writes the
 * converted encoding to an <code>OutputStream</code>.
 *
 * @author Roger Levy
 * @version January 2003
 */
class ConvertEncodingThread extends Thread {
    private OutputStream out = null;
    private InputStream in = null;
    private String from = null;
    private String to = null;

    private Reader r = null;
    private Writer w = null;


    /**
     * Constructor creates the thread.
     * @param in The InputStream to be converted.
     * @param out The OutputStream to write to.
     * @param from The encoding of the InputStream.
     * @param to The desired encoding for the OutputStream.
     * @throws IOException, UnsupportedEncodingException
     */
    public ConvertEncodingThread(InputStream in, OutputStream out, String from, String to) throws IOException, UnsupportedEncodingException {
	this.in = in;
	this.out = out;
	this.from = from;
	this.to = to;

	this.r = new BufferedReader(new InputStreamReader(in, from));
	this.w = new BufferedWriter(new OutputStreamWriter(out, to));
    }

    public void run() {
	if (r != null && w != null) {
	    try {
		char[] buffer = new char[4096];
		int c;
		while((c = r.read(buffer)) != -1)
		    w.write(buffer, 0, c);
		r.close();
		w.flush();
		r.close();
	    }
	    catch (IOException e) {
		System.err.println("ConvertEncodingThread run: " + e);
	    }
	}
    }
    
    /**
     * The main() method converts a file. Its arguments are (Infile,
     * Outfile, EncodingIn, EncodingOut).
     */
    public static void main(String[] args) throws IOException, UnsupportedEncodingException{
	try {
	    BufferedOutputStream fw = new BufferedOutputStream(new FileOutputStream(args[1]));
	    BufferedInputStream fr = new BufferedInputStream(new FileInputStream(args[0]));
	    
	    String from = args[2];
	    String to = args[3];
	    // Use default encoding if no encoding is specified.
	    if (from == null) from = System.getProperty("file.encoding");
	    if (to == null) to = System.getProperty("file.encoding");
	    
	    new ConvertEncodingThread(fr,fw,from,to).start();
	}
	catch(Exception e) {
	    System.out.print(e.getMessage());
	    System.exit(1);
	}
    }

}
