package edu.stanford.nlp.trees.international;

import java.util.*;
import java.io.*;

/** A lexer for the Penn Chinese Treebank.  Supports Chinese characters.
 *  @author Roger Levy
 */

%%

%class CHTBLexer
%standalone
%unicode
%int


%{

  public int IGNORE = 0;
  public int ACCEPT = 1;

  public void pushback(int n) {
    yypushback(n);
  }

  public String match() {
    return yytext();
  }


%}

%state BODY
%state ENDMATERIAL

BODYBEGINS = <BODY>|<HEADLINE>
BODYENDS = <\/BODY>|<[+]
SRCID = <SRCID>[^<]*<\/SRCID>


MISCSGML = <[^+][^>]*>


/* there are a few non-CJK characters as terminals; in particular 'II'
 * shows up as part of a terminal, and '-' and \u00B7 are used in nouns */

CJKCHAR = \u00D7|\uE37B|\u2236|\u25CB|[\u3000-\u303F]|[\u3400-\u4DBF]|[\u4E00-\u9FAF]|[\uFE30-\uFE4F]|[\uFF00-\uFFEF]|[\uFFFD\u0373]
CJK = (II|-|\u00B7|{CJKCHAR})+
ASCIINUMBER = [0-9]
ASCIILETTER = [A-Za-z]


LINEEND = \r|\n|\r\n|\u2028|\u2029|\u000B|\u000C|\u0085
SPACE = [ \t]+

PAREN = \(|\)
LABEL = [A-Za-z0-9*-=&]+
PUNCTMARK = [\u2000-\u206F]|[\,\.\?\!\"\'\`]|[\u00A1-\u00BF]|[\u2500-\u257F]
PUNCT = {PUNCTMARK}+

TERMINAL = ({CJK}|{PUNCTMARK}|{ASCIINUMBER}|{ASCIILETTER})+

%%

<YYINITIAL> {
  {BODYBEGINS}          { yybegin(BODY); 
                          // System.out.println("Beginning of main body");
                          return IGNORE; }
  {SRCID}               { return IGNORE; }
  {LINEEND}	        { return IGNORE; }
  .                     { return IGNORE; }
}

<BODY> {
  {BODYENDS}              { yybegin(ENDMATERIAL); return IGNORE;
                            /* System.out.println("End of main body"); */ }
  {CJK}                   { return ACCEPT; }
  {PAREN}                 { return ACCEPT; }
  {LABEL}                 { return ACCEPT; }  
  {PUNCT}                 { return ACCEPT; }
  {TERMINAL}              { return ACCEPT; }
  {LINEEND}	          { return IGNORE; }
  {MISCSGML}		  { return IGNORE; }
  {SRCID}                 { return IGNORE; }
  {SPACE}                 { return IGNORE; }
  .		{ 	try {
	PrintWriter p = new PrintWriter(new OutputStreamWriter(System.out,
	                                "GB18030"),true);
	p.println("error: \"" + yytext() + "\"");
	} catch (UnsupportedEncodingException e) {}}
    }

<ENDMATERIAL> {
  {LINEEND}	        { return IGNORE; }
  .                     { return IGNORE; }
}
