package edu.stanford.nlp.util;

import java.util.*;

/**
 *  ScoredComparator allows one to compare scored things.
 *
 *  @author Dan Klein
 *  @version 12/4/00
 */
final public class ScoredComparator implements Comparator {

  public static final boolean ASCENDING = true;
  public static final boolean DESCENDING = false;

  public static final ScoredComparator ASCENDING_COMPARATOR =
      new ScoredComparator(ASCENDING);

  public static final ScoredComparator DESCENDING_COMPARATOR =
      new ScoredComparator(DESCENDING);

  private final boolean ascending;

  public ScoredComparator() {ascending = ASCENDING;}

  public ScoredComparator(boolean ascending) {
    this.ascending = ascending;
  }

  public int compare(Object o1, Object o2) {
    if (o1 == o2)
      return 0;
    /*if (! (o1 instanceof Scored) ||
        ! (o2 instanceof Scored))
        throw new ClassCastException();*/
    double d1 = ((Scored)o1).score();
    double d2 = ((Scored)o2).score();
    if(ascending) {
      if (d1 < d2)
        return -1;
      if (d1 > d2)
        return 1;
    } else {
      if (d1 < d2)
        return 1;
      if (d1 > d2)
        return -1;
    }
    return 0;
  }

  public boolean equals(Object o) {
    if (o==this)
      return true;
    if (o instanceof ScoredComparator) {
      ScoredComparator sc = (ScoredComparator) o;
      if (ascending == sc.ascending)
        return true;
    }
    return false;
  }

  /** Return the hashCode: there are only two distinct comparators by
   *  equals().
   */
  public int hashCode() {
    if (ascending) {
      return (1 << 23);
    } else {
      return (1 << 23) + 1;
    }
  }

}
